(require 'jka-compr)
(require 'navi2ch-vars)
(require 'my-navi2ch-article-info)

(defvar my-navi2ch-compr-init-compress-p nil
  "$B?75,$K<hF@$9$k(B dat $B$r(B gzip $B05=L$9$k$+(B")

(defvar my-navi2ch-compr-auto-compress-p nil
  "DAT $BMn$A$7$?%9%l$r<+F0$G05=L$9$k$+(B")

;; (defvar navi2ch-article-file-regexp nil)
;; (setq navi2ch-article-file-regexp "[0-9]+\\.dat\\(\\.gz\\)?\\'")
(defvar navi2ch-article-local-dat-regexp nil)
(setq navi2ch-article-local-dat-regexp "[0-9]+\\.dat\\(\\.gz\\)?\\'")

;;; $B4X?tDj5A(B

(defun my-navi2ch-compr-compressed-file-attributes-size (file)
  "$B05=L%U%!%$%k$N%U%!%$%k%5%$%:$H05=LA0$N%U%!%$%k%5%$%:$rJV$9(B"
  (let ((dat (when (file-exists-p (expand-file-name file))
	       (with-temp-buffer
		 (apply 'call-process
			"gzip" nil t nil
			(list "-l" (expand-file-name file)))
		 (split-string
		  (buffer-substring (point-min) (point-max)))))))
    (when dat
      (mapcar 'string-to-number
	      (list (nth 4 dat) (nth 5 dat))))))

(defun my-navi2ch-compr-auto-compress-article (status header file)
  "$BI,MW$K1~$8$F%9%l$r05=L$9$k(B"
  (when (and my-navi2ch-compr-auto-compress-p
	     (or (eq status 'not-found)
		 (eq status 'moved)
		 (navi2ch-net-get-state 'kako header))
	     (not (string-match "\\.gz\\'" file)))
    (my-navi2ch-compr-compressed-file file)))

(defun my-navi2ch-compr-compressed-file (file)
  "$B%U%!%$%k$r(B gzip $B$G05=L$9$k(B"
  (when (file-exists-p file)
    (if (eq 0
	    (with-temp-buffer
	      (message "Compressed %s..." file)
	      (apply 'call-process
		     "gzip" nil t nil
		     (list "-f" (expand-file-name file)))))
	(message "%sdone." (current-message))
      (message "%serror!" (current-message)))))

(defun my-navi2ch-compr-decompressed-file (file)
  "$B%U%!%$%k$r2rE`$9$k(B"
  (when (file-exists-p file)
    (if (eq 0
	    (with-temp-buffer
	      (message "Decompressed %s..." file)
	      (apply 'call-process
		     "gzip" nil t nil
		     (list "-d" (expand-file-name file)))))
	(message "%sdone." (current-message))
      (message "%serror!" (current-message)))))

(defun my-navi2ch-compr-toggle-compress-file (&optional flag)
  "$B%U%!%$%k$N05=L!"Hs05=L$r@Z$jBX$($k(B"
  (let* ((item (navi2ch-bm-get-property-internal (point)))
	 (board (navi2ch-bm-get-board-internal item))
	 (article (navi2ch-bm-get-article-internal item))
	 (file (navi2ch-article-get-file-name board article)))
    (if (string-equal (file-name-extension file) "gz")
	(unless (and flag (> flag 0))
	  (my-navi2ch-compr-decompressed-file file))
      (unless (and flag (< flag 0))
	(my-navi2ch-compr-compressed-file file)))))

(defun my-navi2ch-compr-compress-article (&optional flag)
  "$B%U%!%$%k$N05=L!"Hs05=L$r@Z$jBX$($k(B"
  (interactive "P")
  (my-navi2ch-compr-toggle-compress-file (when flag
					   (prefix-numeric-value flag)))
  (my-navi2ch-article-info-redraw-line))

(defun my-navi2ch-compr-compress-mark-article (&optional flag)
  "$B%U%!%$%k$N05=L!"Hs05=L$r@Z$jBX$($k(B"
  (interactive "P")
  (navi2ch-bm-exec-subr 'my-navi2ch-compr-compress-article flag))

;;; $B%-!<Dj5A$NDI2C(B
(mapcar
 (lambda (x)
   (let ((func 'define-key))
     (funcall func x "Z" 'my-navi2ch-compr-compress-article)
     (funcall func x "mZ" 'my-navi2ch-compr-compress-mark-article)))
 `(,navi2ch-bookmark-mode-map
   ,navi2ch-history-mode-map
   ,navi2ch-articles-mode-map
   ,navi2ch-search-mode-map))

;;; advice $BDj5A(B
(defadvice my-navi2ch-article-info-transfer-size
  (around my-navi2ch-compr activate)
  (let* ((file (navi2ch-article-get-file-name
		(navi2ch-bm-get-board-internal (ad-get-arg 0))
		(navi2ch-bm-get-article-internal (ad-get-arg 0))))
	 (size (if (string-equal (file-name-extension file) "gz")
		   (my-navi2ch-compr-compressed-file-attributes-size file)
		 (nth 7 (file-attributes file))))
	 (width2 (ad-get-arg 1)))
    (setq ad-return-value
	  (if (listp size)
	      (format (concat "%" width "fk" " (%" width "fk)")
		      (/ (nth 0 size) 1024.0) (/ (nth 1 size) 1024.0))
	    (format (concat "%" width "fk")
		    (if size (/ size 1024.0) 0))))))

(defadvice navi2ch-article-get-file-name (after my-navi2ch-compr activate)
  "gzip $B$G05=L$5$l$?%U%!%$%k$,$"$k>l9g$O05=L$5$l$?%U%!%$%kL>$rJV$9(B"
  (let* ((file (if (string-equal (file-name-extension ad-return-value) "gz")
		   (file-name-sans-extension ad-return-value)
		 ad-return-value))
	 (compr-file (concat file ".gz")))
    (setq ad-return-value
	  (cond
	   ((file-exists-p file) file)
	   ((file-exists-p compr-file) compr-file)
	   (my-navi2ch-compr-init-compress-p compr-file)
	   (t file)))))

;; (defadvice navi2ch-file-attributes (after my-navi2ch-compr activate)
;;   "file $B$,05=L$5$l$F$$$k>l9g$O!"05=LA0$N%U%!%$%k%5%$%:$rJV$9(B"
;;   (let ((file (ad-get-arg 0)))
;;     (when (string-equal (file-name-extension file) "gz")
;;       (let ((size
;; 	     (car (cdr
;; 		   (my-navi2ch-compr-compressed-file-attributes-size file)))))
;; 	(when size
;; 	  (setcar (nthcdr 7 ad-return-value) size))))))
(defadvice navi2ch-file-size (around my-navi2ch-compr activate)
  "file $B$,05=L$5$l$F$$$k>l9g$O!"05=LA0$N%U%!%$%k%5%$%:$rJV$9(B"
  (let ((file (ad-get-arg 0)))
    (if (not (string-equal (file-name-extension file) "gz"))
	ad-do-it
      (let ((size
	     (car (cdr
		   (my-navi2ch-compr-compressed-file-attributes-size file)))))
	(if size
	    (setq ad-return-value size)
	  ad-do-it)))))

;; $B%U%!%$%k$,05=L$5$l$F$$$F!"(Bbigin $B$,;XDj$5$l$F$$$k>l9g$O(B bigin $B$r>.$5$/$7(B
;; $BM>J,$K%U%!%$%k$NFbMF$r%P%C%U%!$KFI9~$_!"L5BL$JJ8;zNs$r:o=|$9$k(B
(defadvice navi2ch-insert-file-contents
  (around my-navi2ch-trim-message-list disable)
  (if (not (string-match "\\.gz\\'" file))
      ad-do-it
    (let ((begin_tmp (ad-get-arg 1)))
      (ad-set-arg 1 (and begin_tmp (if (< begin_tmp 32) 0 (- begin_tmp 32))))
      ad-do-it
      (save-excursion
	(goto-char (point-min))
	(re-search-forward ".*<>.*<>.*<>.*<>.*$" nil t)
	(delete-region (point-min) (navi2ch-line-beginning-position))))))

;; navi2ch-insert-file-contents() $B$N(B my-navi2ch-trim-message-list $BCGJR$r(B
;; $B3h@-2=$9$k(B
(defadvice navi2ch-article-get-message-list (around my-navi2ch-compr activate)
  (unwind-protect
      (progn
	(ad-enable-advice 'navi2ch-insert-file-contents
			  'around 'my-navi2ch-trim-message-list)
	(ad-activate 'navi2ch-insert-file-contents)
	ad-do-it)
    (ad-disable-advice 'navi2ch-insert-file-contents
		       'around 'my-navi2ch-trim-message-list)
    (ad-activate 'navi2ch-insert-file-contents)))

;; (defadvice navi2ch-file-name-sans-extension (around my-navi2ch-compr activate)
;;   "$BA4$F$N3HD%;R$r=|$$$?%U%!%$%kL>$rJV$9(B"
;;   (setq ad-return-value
;; 	(if (file-name-extension (ad-get-arg 0))
;; 	    (navi2ch-file-name-sans-extension
;; 	     (file-name-sans-extension (ad-get-arg 0)))
;; 	  (file-name-sans-extension file))))
(defadvice navi2ch-article-file-name-to-artid
  (around my-navi2ch-compr activate)
  "$BA4$F$N3HD%;R$r=|$$$?%U%!%$%kL>$rJV$9(B"
  (setq ad-return-value
	(if (file-name-extension (ad-get-arg 0))
	    (navi2ch-article-file-name-to-artid
	     (file-name-sans-extension (ad-get-arg 0)))
;;	  (file-name-sans-extension file))))
	  (file-name-sans-extension (ad-get-arg 0)))))

(unless (fboundp 'with-auto-compression-mode)
  (defmacro with-auto-compression-mode (&rest body)
    "Evalute BODY with automatic file compression and uncompression enabled."
    (let ((already-installed (make-symbol "already-installed")))
      `(let ((,already-installed (jka-compr-installed-p)))
	 (unwind-protect
	     (progn
	       (unless ,already-installed
		 (jka-compr-install))
	       ,@body)
	   (unless ,already-installed
	     (jka-compr-uninstall)))))))

;; with-auto-compression-mode() $B$NIU2C(B
(mapcar
 (lambda (x)
   (let ((func `(lambda ()
		  (defadvice ,x (around my-navi2ch-compr activate)
		    (with-auto-compression-mode ad-do-it)))))
     (funcall func)))
 '(navi2ch-insert-file-contents
   navi2ch-net-update-file
   navi2ch-net-update-file-diff
   navi2ch-net-update-file-with-readcgi
   navi2ch-oyster-update-file-with-offlaw
   navi2ch-count-lines-file
   navi2ch-localfile-update-file))

;; (defadvice navi2ch-insert-file-contents (around my-navi2ch-compr activate)
;;   (with-auto-compression-mode ad-do-it))


(provide 'my-navi2ch-compr)
