;;;
;;; global-bookmark $B$NJB$Y$+$($r2DG=$K$9$k(B
;;;
(defvar my-navi2ch-bookmark-list-stack nil)

(defun my-navi2ch-list-kill-global-bookmark ()
  (interactive)
  (let ((board (get-text-property (point) 'board)))
    (if (eq (cdr (assq 'type board)) 'bookmark)
	(my-navi2ch-list-kill-global-bookmark-subr (cdr (assq 'id board)))
      (message "This line is not bookmark!"))))

(defun my-navi2ch-list-kill-global-bookmark-subr (bookmark-id)
  (let ((bookmark (assoc bookmark-id navi2ch-bookmark-list)))
    (beginning-of-line)
    (push bookmark my-navi2ch-bookmark-list-stack)
    (setq navi2ch-bookmark-list (delete bookmark navi2ch-bookmark-list))
    (set-buffer navi2ch-list-buffer-name)
    (navi2ch-list-sync-global-bookmark-category)
    (navi2ch-bookmark-save-info)))

(defun my-navi2ch-list-yank-global-bookmark ()
  (interactive)
  (let ((board (get-text-property (point) 'board)))
    (if (eq (cdr (assq 'type board)) 'bookmark)
	(my-navi2ch-list-yank-global-bookmark-subr (cdr (assq 'id board)))
      (message "This line is not bookmark!"))))

(defun my-navi2ch-list-yank-global-bookmark-subr (bookmark-id)
  (let ((pair (pop my-navi2ch-bookmark-list-stack)))
    (if pair
	(progn
	  (beginning-of-line)
	  (let ((list (member (assoc bookmark-id navi2ch-bookmark-list)
			      navi2ch-bookmark-list)))
	    (setcdr list (copy-sequence list))
	    (setcar list pair)
	    (set-buffer navi2ch-list-buffer-name)
	    (navi2ch-list-sync-global-bookmark-category))
	  (navi2ch-bookmark-save-info))
      (message "stack is empty"))))

(define-key navi2ch-list-mode-map "\C-k" 'my-navi2ch-list-kill-global-bookmark)
(define-key navi2ch-list-mode-map "\C-y" 'my-navi2ch-list-yank-global-bookmark)

;;;
;;; article $B$N(B URL $B>e$G(B A $B$r2!$9$3$H$G(B global-bookmark $B$KDI2C2DG=$K$9$k(B
;;;
(defun my-navi2ch-add-global-bookmark (bookmark-id)
  "point $B$,(B 2ch $B$N(B article $B$X$N(B URL $B$J$i(B global-bookmark $B$KDI2C(B"
  (interactive (list (navi2ch-bookmark-read-id "bookmark id: ")))
  (let ((url (get-text-property (point) 'navi2ch-link)))
    (if (not url)
	(message "This point is not URL!")
      (let ((board (navi2ch-board-url-to-board url))
	    (article (navi2ch-article-url-to-article url)))
	(if (and board article)
	    (navi2ch-bookmark-add bookmark-id board article)
	  (message "This URL is not 2ch's article!"))))))

(defadvice navi2ch-article-add-global-bookmark
  (around url-add-global-bookmark activate)
  (let ((url (get-text-property (point) 'navi2ch-link)))
    (if (not url)
	ad-do-it
      (let ((board (navi2ch-board-url-to-board url))
	    (article (navi2ch-article-url-to-article url)))
	(if (and board article)
	    (navi2ch-bookmark-add bookmark-id board article)
	  ad-do-it)))))

;;;
;;; $BHD0\E>$7$F$b!"2a5n%9%l$r3+$1$k$h$&$K$9$k(B
;;;
(defun my-navi2ch-replace-url (url)
  "2ch $B$N(B url $B$rCV$-49$($k(B"
  (let* ((id (cdr (assoc 'id  (navi2ch-board-url-to-board url))))
	 (board (navi2ch-board-url-to-board url))
	 (article (navi2ch-article-url-to-article url))
	 (cache (and board (navi2ch-article-check-cached board article)))
	 (host (navi2ch-url-to-host url))
	 ;; 		 (info (navi2ch-article-load-info board article))
	 ;; 		 (kako (or (cdr (assoc 'kako info))
	 ;; 				   (cdr (assoc 'moved info))))
	 (range (save-match-data
		  (when (string-match
			 "/\\([0-9]*\\)\\(-?\\)\\([0-9]*\\)\\(n?\\)\\'" url)
		    (list (match-string 1 url)
			  (match-string 3 url)
			  (match-string 4 url)))))
	 (start (unless (zerop (length (nth 0 range)))
		  (string-to-number (nth 0 range))))
	 (end (unless (zerop (length (nth 1 range)))
		(string-to-number (nth 1 range))))
	 (nofirst (not (zerop (length (nth 2 range)))))
	 (lst (mapcar (lambda (x)
			(cons (cdr (assq 'id x))
			      (cdr (assq 'uri x))))
		      navi2ch-list-board-name-list))
	 (replace (cdr (assoc id lst)))
	 (frm "Cache $B$,(B %s/ $B$KM-$j$^$9!#(Burl $B$N%[%9%H$rCV$-49$($^$9$+(B?")
	 rtn)
    (if cache ;; cache $B$,M-$k$J$i(B navi2ch-url-to-host $B$K(B host $B$rDI2C(B
	(unless (member host navi2ch-2ch-host-list)
	  (setq navi2ch-2ch-host-list (cons host navi2ch-2ch-host-list)))
      (when (and board article replace)
	(let ((slot (assoc 'uri board)))
	  (when slot
	    (setcdr slot replace)))
	(if (or navi2ch-offline	;; offline $B$J$i6/@)E*$KFI$_BX$($k(B
		(and (navi2ch-article-check-cached board article)
		     (y-or-n-p (format frm host))))
	    (setq rtn (navi2ch-article-to-url board
					      article start end nofirst))
	  ;; cache $B$,L5$/(B online $B$J$i(B navi2ch-url-to-host $B$K(B host $B$rDI2C(B
	  (unless (member host navi2ch-2ch-host-list)
	    (setq navi2ch-2ch-host-list (cons host
					      navi2ch-2ch-host-list))))))
    rtn))

(defadvice navi2ch-article-select-current-link-url
  (before replace-url activate)
  ;;   (unless (or (ad-get-arg 1)
  ;; 			  (navi2ch-2ch-url-p url))
  (unless (ad-get-arg 1)
    (let ((tmp (my-navi2ch-replace-url url)))
      (when tmp
	(ad-set-arg 0 tmp)))))

;;;
;;; $BB8:_$7$J$$(B artid $B$r(B bookmark $B$H(B hide $B$N>pJs$+$i:o=|$9$k(B
;;;
(defun my-navi2ch-board-trim-bookmarn-n-hide-data ()
  (interactive)
  (mapcar (lambda (x)
	    (mapcar (lambda (y)
		      (unless (assoc y navi2ch-board-subject-alist)
			(delete y x)))
		    (cdr x)))
	  (mapcar (lambda (x) (assoc x navi2ch-board-current-board))
		  '(bookmark hide))))

(add-hook 'navi2ch-board-after-sync-hook
	  'my-navi2ch-board-trim-bookmarn-n-hide-data)

;;;
;;; hide $B$7$?%l%9$K%l%9$7$?%l%9$r(B hide $B$9$k(B
;;;
(defun my-navi2ch-aborn-chain-subr (str)
  "NG $B%l%9$KBP$9$k%"%s%+!<$,4^$^$l$k%l%9$J$i(B NG $B$H$9$k(B"
  (let ((regexp (concat navi2ch-article-number-prefix-regexp
			navi2ch-article-number-number-regexp))
	(start 0))
    (catch 'hit
      (while (string-match regexp str start)
	(let* ((hit-str (match-string 0 str))
	       (tmp (navi2ch-article-get-number-list (match-string 1 str)))
	       (num-lst (if (listp tmp) tmp (list tmp))))
	  (setq start (+ start (length hit-str)))
	  (mapcar (lambda (num)
		    (cond
		     ((member num (assq 'hide navi2ch-article-current-article))
		      ;;(setq ad-return-value 'hide)
		      (throw 'hit 'hide))
		     ((assq num
			    (assq 'replace
				  navi2ch-article-message-filter-cache))
		      ;;(setq ad-return-value
		      ;;    (concat "$B$"$\!<$s$NO":?(B " hit-str))
		      ;;(setq ad-return-value "$B$"$\!<$s$NO":?(B")
		      (throw 'hit '"$B$"$\!<$s$NO":?(B"))
		     (t nil)))
		  num-lst))))))

(defadvice navi2ch-article-message-filter-by-message
  (around my-navi2ch-aborn-chain activate)
  (let ((str (cdr (assoc 'data (ad-get-arg 0)))))
    ad-do-it
    (unless (or (eq ad-return-value 'hide)
		(stringp ad-return-value))
      (let ((rtn (my-navi2ch-aborn-chain-subr str)))
	(when (or (eq rtn 'hide) (stringp rtn))
	  (setq ad-return-value rtn))))))

(defun my-navi2ch-article-hide-message-chain ()
  "current $B$N%l%9$KBP$9$k%l%9$r(B hide $B$9$k(B"
  (let ((regexp (concat navi2ch-article-number-prefix-regexp
			navi2ch-article-number-number-regexp))
	(number (navi2ch-article-get-current-number)))
    (save-excursion
      (while (re-search-forward regexp nil t)
	(goto-char (1- (point)))
	(let* ((tmp (navi2ch-article-get-number-list (match-string 1)))
	       (num-lst (if (listp tmp) tmp (list tmp))))
	  (catch 'hit
	    (mapcar (lambda (num)
		      (when (eq num number)
			(navi2ch-article-hide-message)
			(throw 'hit 'hide)))
		  num-lst)))))))

(defun my-navi2ch-article-cancel-hide-message-chain ()
  "current $B$N%l%9$KBP$9$k%l%9$N(B hide $B$r(B cancel $B$9$k(B"
  (let ((regexp (concat navi2ch-article-number-prefix-regexp
			navi2ch-article-number-number-regexp))
	(number (navi2ch-article-get-current-number)))
    (save-excursion
      (while (re-search-forward regexp nil t)
	(goto-char (1- (point)))
	(let* ((tmp (navi2ch-article-get-number-list (match-string 1)))
	       (num-lst (if (listp tmp) tmp (list tmp))))
	  (catch 'hit
	    (mapcar (lambda (num)
		      (when (eq num number)
			(navi2ch-article-cancel-hide-message)
			(throw 'hit 'cancel)))
		  num-lst)))))))

;; hook $B$N@_Dj(B
(add-hook 'navi2ch-article-hide-message-hook
	  'my-navi2ch-article-hide-message-chain)
(add-hook 'navi2ch-article-cancel-hide-message-hook
	  'my-navi2ch-article-cancel-hide-message-chain)

(defun my-navi2ch-show-ng-count ()
  "NG $B$J%l%9?t$rI=<($9$k(B"
  (interactive)
  (let* ((hide (delq nil (cdr (assq 'hide navi2ch-article-current-article))))
	 (replace
	  (delq nil
		(mapcar 'car
			(cdr (assq 'replace
				   navi2ch-article-message-filter-cache)))))
	 (h-cnt (or (length hide) 0))
	 (r-cnt (or (length replace) 0)))
    (message "NG: %d res (hide: %d, replace: %d)"
	     (+ h-cnt r-cnt) h-cnt r-cnt)))

;;;
;;; URL $B$r(B decode $B$7$F(B mini-buffer $B$KI=<($9$k(B
;;;
(defun my-navi2ch-w3m-url-decode-string (str &optional coding)
  "`w3m-url-decode-string' $B$r%j%M!<%`$7Hy=$@5$7$?$b$N(B"
  (let ((start 0)
	(buf))
    (while (string-match "+\\|%\\(0D%0A\\|\\([0-9a-fA-F][0-9a-fA-F]\\)\\)"
			 str start)
      (push (substring str start (match-beginning 0)) buf)
      (push (cond
	     ((match-beginning 2)
	      (vector (string-to-number (match-string 2 str) 16)))
	     ((match-beginning 1) "\n")
	     (t " "))
	    buf)
      (setq start (match-end 0)))
    (decode-coding-string
     (apply 'concat (nreverse (cons (substring str start) buf)))
     (or coding
	 'utf-8))))

(defadvice navi2ch-article-get-link-text (after decode-url-string activate)
  (unless ad-return-value
    (setq ad-return-value
	  (let* ((point (or point (point)))
		 (type (get-text-property point 'navi2ch-link-type))
		 (prop (get-text-property point 'navi2ch-link)))
	    (when (eq type 'url)
	      (my-navi2ch-w3m-url-decode-string
	       (navi2ch-ifxemacs
		   (identity prop)
		 (string-as-unibyte prop))
	       (cond
		((string-match "&ie=UTF-8" prop) 'utf-8) ;google
		(t 'undecided))))))))

;;;
;;; $B@55,I=8=$K%^%C%A$7$?%F%-%9%H$r%O%$%i%$%H$9$k(B
;;;
(defvar my-navi2ch-highlight-message-regexp-list nil
  "face $B$r@_Dj$7$?$$@55,I=8=$H(B face $B$N%j%9%H(B
 $B@_DjNc(B
  '((\"meadow\\\\|emacs\" 'font-lock-warning-face)
    (\"vi\" 'font-lock-comment-delimiter-face))")

(defun my-navi2ch-highlight-message ()
  (interactive)
  (let ((lst my-navi2ch-highlight-message-regexp-list))
    (while lst
      (goto-char (point-min))
      (let ((regexp (car (car lst)))
	    (face (car (cdr (car lst)))))
	(while (re-search-forward regexp nil t)
	  (add-text-properties
	   (match-beginning 0) (match-end 0) `(face ,face))))
      (setq lst (cdr lst)))))

(add-hook 'navi2ch-article-arrange-message-hook
	  'my-navi2ch-highlight-message)

;;;
;;; $BHD$r(B sync $B$7$?;~$K!"FCDj$N%9%l$r%0%m!<%P%k%V%C%/%^!<%/$KDI2C$9$k(B
;;;
(defvar my-navi2ch-add-bookmark-rule nil
  "`my-navi2ch-add-bookmark' $B$G%0%m!<%P%k%V%C%/%^!<%/$KDI2C$9$k>r7o$N%j%9%H(B
$B%j%9%HFb$NMWAG(B
  1st: $B%0%m!<%P%k%V%C%/%^!<%/$KEPO?$9$k%5%V%8%'%/%H$K%^%C%A$9$k@55,I=8=(B
  2nd: $BEPO?@h$N%0%m!<%P%k%V%C%/%^!<%/(B id
  3rd: $B%k!<%k$rE,MQ$9$kHD(B id $B$N%j%9%H(B (nil $B$J$iA4$F$NHD$KE,MQ$9$k(B)
  4th: $B%k!<%k$rE,MQ$7$J$$HD(B id $B$N%j%9%H(B (nil $B$J$i=|30$O$7$J$$(B)
       nil $B0J30$N>l9g$O(B 3rd $B$NMWAG$KM%@h$9$k(B
$B@_DjNc(B
'((\"emacs\\\\|vi\" \"bm-editor\" nil (\"software\"))
  (\"cvs\\\\|subversion\" \"bm-scm\" (\"unix\" \"linux\") nil))")

(defvar my-navi2ch-add-bookmark-check-all-article nil
  "$B?75,$N%9%l0J30$b(B `my-navi2ch-add-bookmark' $B$G%A%'%C%/$9$k$+(B
t  : $BA4%9%l$r%A%'%C%/(B
nil: $B?75,$N%9%l$N$_%A%'%C%/(B")

(defun my-navi2ch-add-bookmark-rule-make ()
  "`my-navi2ch-add-bookmark-rule' $B$+$iEPO?$N8=:_$NHDMQ$N%k!<%k$r:n$k(B"
  (let ((board-id (cdr (assoc 'id navi2ch-board-current-board)))
	(rule-lst my-navi2ch-add-bookmark-rule)
	new-rule)
    (while rule-lst
      (let ((rule (car rule-lst)))
	(setq rule-lst (cdr rule-lst))
	(when (and
	       (not (member board-id (nth 3 rule)))
	       (or (not (nth 2 rule))
		   (member board-id (nth 2 rule)))
	       (assoc (nth 1 rule) navi2ch-bookmark-list))
	  (setq new-rule (cons rule new-rule)))))
    new-rule))

(defun my-navi2ch-add-bookmark ()
  "`my-navi2ch-add-bookmark-rule' $B$K=>$$%0%m!<%P%k%V%C%/%^!<%/$K%9%l$rDI2C$9$k(B"
  (interactive)
  (unless navi2ch-offline
    (let ((lst (my-navi2ch-add-bookmark-rule-make))
	  flg)
      (save-excursion
	(while lst
	  (let* ((rule (car lst))
		 (regexp (car rule))
		 (bookmark-id (car (cdr rule))))
	    (setq lst (cdr lst))
	    (goto-char (point-min))
	    (while (re-search-forward regexp nil t)
	      (when (or my-navi2ch-add-bookmark-check-all-article
			(progn
			  (navi2ch-bm-goto-updated-mark-column)
			  (looking-at "%")))
		(let* ((item (navi2ch-bm-get-property-internal (point)))
		       (board (navi2ch-bm-get-board-internal item))
		       (article (navi2ch-bm-get-article-internal item)))
		  (when item
		    ;; global bookmark $B$KDI2C(B
		    (let ((bookmark (assoc bookmark-id navi2ch-bookmark-list)))
		      (setcdr (cdr bookmark)
			      (navi2ch-put-alist
			       (navi2ch-bookmark-get-key board article)
			       (list (cons 'board board)
				     (cons 'article article))
			       (cdr (cdr bookmark)))))
		    (setq flg t)
		    ;; my-navi2ch-article-info-global-bookmark-alist $B$N99?7(B
		    (when (and
			   (boundp
			    'my-navi2ch-article-info-global-bookmark-alist)
			   my-navi2ch-article-info-global-bookmark-alist
			   (eq major-mode 'navi2ch-board-mode))
		      (let* ((data-list
			      (assoc
			       (cdr (assoc 'id board))
			       my-navi2ch-article-info-global-bookmark-alist))
			     (article-list (cdr (cdr data-list)))
			     (artid (assoc 'artid article))
			     (pos
			      ;;position $B$N<hF@(B
			      (let ((id (cdr artid))
				    (subject-list navi2ch-board-subject-alist))
				(let ((total (length subject-list))
				      (pos (length (member
						    (assoc id subject-list)
						    subject-list))))
				  (if (not pos) nil (- (1+ total) pos)))))
			     (data `(,artid (update . nil) (position . ,pos))))
			(unless (assoc artid article-list)
			  (if article-list
			      (setcdr article-list
				      (cons data (cdr article-list)))
			    (setcdr data-list `((,data))))))))))
	      (forward-line))))
	(when flg
	  (navi2ch-bookmark-save-info))))))

(add-hook 'navi2ch-board-after-sync-hook
	  'my-navi2ch-add-bookmark)

;;;
;;; filter $B$N:GE,2=$r<B;\(B
;;;
(defvar my-navi2ch-filter-elements-optimize-subr-limit 300)
(defun my-navi2ch-filter-elements-optimize-subr (alist)
  "$BJ8;zNs$K%^%C%A$9$k>r7o$rCj=P$7(B regexp $B%^%C%A$KJQ49$9$k(B.
Filter $B>r7o$N$&$A!"J8;zNs$K%^%C%A$+$D(B hide or important $B$rE,MQ$9$k>r7o$r(B
$BCj=P$7!"$=$l$i$r(B regexp $B%^%C%A$N>r7o$KJQ49$9$k(B.
$BJ8;zNs%^%C%A0J30$N>r7o$O!"$=$N$^$^$N>uBV$r0];}(B."
  (let* ((handle '((hide) (important)))
	 (arg (delq nil
		    (mapcar (lambda (x)
			      (let ((slot (assoc (cdr x) handle))
				    rtn)
				(if (and slot (stringp (car x)))
				    (setcdr slot (cons (car x) (cdr slot)))
				  (setq rtn x))
				rtn))
			    alist)))
	 (lst))
    (mapcar (lambda (x)
	      (let* ((top `(,(car x)))
		     (limit my-navi2ch-filter-elements-optimize-subr-limit))
		(while (> (length x) (1+ limit))
		  (let ((len (length x)))
		  (add-to-list 'lst (append top (nthcdr (- len limit) x)))
		  (setcdr (member (nth (- len (1+ limit)) x) x) nil)))
	      (add-to-list 'lst x)))
	  handle)
    (delq nil
	  (append arg
		  (mapcar (lambda (x)
			    (prog1
				(when (cdr x)
				  (cons
				   (list (regexp-opt (cdr x)) 'r) (car x)))
			      (setcdr x nil)))
			  lst)))))

(defvar my-navi2ch-filter-elements-optimize-cache nil
  "$B:GE,2=$7$?(B Filter $B>r7o$rJ];}$9$k(B.")
(defun my-navi2ch-filter-elements-optimize (&optional refresh)
  "Filter $B>r7o$r:GE,2=$9$k(B.
cache $B$,$"$l$P(B cache $B$rJV$9(B.
$BC"$7!"(Breflesh $B$,(B non-nil $B$G$"$l$P(B cache $B$r;HMQ$7$J$$(B."
  (if (and (not refresh) my-navi2ch-filter-elements-optimize-cache)
      my-navi2ch-filter-elements-optimize-cache
    (let ((al '((name . navi2ch-article-message-filter-by-name-alist)
		(mail . navi2ch-article-message-filter-by-mail-alist)
		(id . navi2ch-article-message-filter-by-id-alist)
		(message . navi2ch-article-message-filter-by-message-alist)
		(subject . navi2ch-article-message-filter-by-subject-alist)))
	  (cache '((name) (mail) (id) (message) (subject))))
      (mapcar (lambda (x)
		(setcdr x (my-navi2ch-filter-elements-optimize-subr
			   (symbol-value (cdr (assoc (car x) al))))))
	      cache)
    (setq my-navi2ch-filter-elements-optimize-cache cache))))

(defadvice navi2ch-article-apply-message-filters (around
						  filter-optimize activate)
  (let* ((filter (my-navi2ch-filter-elements-optimize))
	 (navi2ch-article-message-filter-by-name-alist
	  (cdr (assoc 'name filter)))
	 (navi2ch-article-message-filter-by-mail-alist
	  (cdr (assoc 'mail filter)))
	 (navi2ch-article-message-filter-by-id-alist
	  (cdr (assoc 'id filter)))
	 (navi2ch-article-message-filter-by-message-alist
	  (cdr (assoc 'message filter)))
	 (navi2ch-article-message-filter-by-subject-alist
	  (cdr (assoc 'subject filter))))
	ad-do-it)
  (if (and navi2ch-article-sort-message-filter-rules
	   (member ad-return-value '(hide important)))
      ;; filter $B>r7o$N%=!<%H$r<B;\$9$k$?$a!"DL>o$N%U%#%k%?=hM}$r<B;\(B
      ad-do-it))

(defadvice navi2ch-auto-modify-variables
  (after filter-optimize activate)
  (let* ((variable (car (ad-get-arg 0)))
	 (element
	      (cdr
	       (assoc
		variable
		'((navi2ch-article-message-filter-by-name-alist . name)
		  (navi2ch-article-message-filter-by-mail-alist . mail)
		  (navi2ch-article-message-filter-by-id-alist . id)
		  (navi2ch-article-message-filter-by-message-alist . message)
		  (navi2ch-article-message-filter-by-subject-alist . subject)))))
	     (slot (assoc element
			  my-navi2ch-filter-elements-optimize-cache)))
	(when slot
	  (setcdr slot (my-navi2ch-filter-elements-optimize-subr
			(symbol-value variable))))))

;;;
;;; ID $B$K(B link $B$rIU2C$9$k(B
;;;
(defvar my-navi2ch-article-hissi-id-limit 4
  "ID $B$K(B `my-navi2ch-article-header-hissi-face' $B$rIU2C$9$k%l%9?t$NogCM(B.")

(defface my-navi2ch-article-header-hissi-face
  '((((class color) (background light)) (:foreground "red"))
    (((class color) (background dark)) (:foreground "red")))
  "$BI,;`!)$J(B ID $B$N(B face")

(defvar my-navi2ch-article-id-hash-table nil
  "ID $B$H%l%9HV$N%F!<%V%k(B.")
(make-variable-buffer-local
 'my-navi2ch-article-id-hash-table)

(defun my-navi2ch-article-update-id-hash-table (dat)
  "hash-table $B$K(B ID $B$H$=$N(B ID $B$N%l%9HV$N%j%9%H$rEPO?$9$k(B."
  (let ((number (car dat))
	(date (cdr (assq 'date (if (listp (cdr dat))
				   (cdr dat)
				 (navi2ch-article-parse-message (cdr dat)))))))
    (when (string-match "ID:\\([a-zA-Z0-9+/]+\\)" date)
      (let ((ID (match-string 1 date)))
	(unless my-navi2ch-article-id-hash-table
	  ;; Table $B$N=i4|2=(B
	  (setq my-navi2ch-article-id-hash-table
		(make-hash-table :size 1000 :test 'equal)))
	(let ((num-lst (gethash
			ID my-navi2ch-article-id-hash-table)))
	  (when (or (not num-lst) (not (member number num-lst)))
	    (setq num-lst (append num-lst (list number)))
	    (puthash ID num-lst my-navi2ch-article-id-hash-table)))))))


(defadvice navi2ch-article-insert-messages (before update-id-hash activate)
  (let ((list (ad-get-arg 0)))
    (mapcar
       'my-navi2ch-article-update-id-hash-table
     list)))

(defun my-navi2ch-article-header-format-function (number name mail date)
  "$B%X%C%@$r%U%)!<%^%C%H$9$k4X?t!#(B
$B%X%C%@$N(B face $B$rIU$1$k$N$b$3$3$G!#(B"
  (when (string-match (concat "\\`" navi2ch-article-number-number-regexp
			      "\\'")
		      name)
    (navi2ch-article-set-link-property-subr (match-beginning 0)
					    (match-end 0)
					    'number
					    (match-string 0 name)
					    name))
  (let (hissi-flg)
    (when (string-match "ID:\\([a-zA-Z0-9+/]+\\)" date)
      (let* ((s (match-beginning 1))
	     (e (match-end 1))
	     (str (match-string 1 date))
	     (num-lst (gethash str my-navi2ch-article-id-hash-table))
	     (len (length num-lst)))
	(when (>= len my-navi2ch-article-hissi-id-limit)
	  (setq hissi-flg t))
	(when (>= len 2)
;; 	  (add-text-properties s e
;; 			       (list 'face 'navi2ch-article-link-face
;; 				     'help-echo #'navi2ch-article-help-echo
;; 				     'link t
;; 				     'mouse-face navi2ch-article-mouse-face
;; ;; number $B$K(B ID $B$r@_Dj$7$F$b%j%s%/$,@5>o$K5!G=$9$k(B
;; ;;				     'number str
;; 				     'number (mapconcat 'number-to-string
;; 							 num-lst ","))
;; 			       date)
	  (add-text-properties s e
			       (list 'face 'navi2ch-article-link-face
				     'help-echo #'navi2ch-article-help-echo
				     'mouse-face navi2ch-article-mouse-face
				     'hissi t
				     'navi2ch-link-type 'number
				     'navi2ch-link (mapconcat 'number-to-string
							      num-lst ","))
			       date)
	  )))
    (let ((from-header (propertize "From: "
				   'face 'navi2ch-article-header-face))
	  (from (propertize (concat (format "[%d] " number)
				    name
				    (format " <%s>\n" mail))
			    'face 'navi2ch-article-header-contents-face))
	  (date-header (propertize "Date: "
				   'face 'navi2ch-article-header-face))
	  (date
	   (if (not hissi-flg)
	       (propertize (funcall
			    (if (boundp 'navi2ch-article-date-format-function)
				navi2ch-article-date-format-function
			      'identity)
			    date)
			   'face
			   'navi2ch-article-header-contents-face)
	     (string-match "\\(.+ID:\\)\\([a-zA-Z0-9+/]+\\)" date)
	     (let ((day (match-string 1 date))
		   (id (match-string 2 date)))
	       (concat (propertize day
				   'face 'navi2ch-article-header-contents-face)
		       (propertize id
				   'face 'my-navi2ch-article-header-hissi-face)
		       ))))
	  (start 0) next)
      (while start
	(setq next
	      (next-single-property-change start 'navi2ch-fusianasan-flag from))
	(when (get-text-property start 'navi2ch-fusianasan-flag from)
	  (add-text-properties start (or next (length from))
			       '(face navi2ch-article-header-fusianasan-face)
			       from))
	(setq start next))
      (concat from-header from date-header date "\n\n"))))

(setq navi2ch-article-header-format-function
      'my-navi2ch-article-header-format-function)

(defadvice navi2ch-article-next-link
  (around check-hissi activate)
  (let ((start (point))
	(point (navi2ch-next-property (point) 'navi2ch-link)))
    (while point
      (goto-char point)
      (setq point
	    (cond
	     ((not (get-text-property (point) 'hissi)) nil)
	     ((get-text-property (point) 'hissi)
	      (or (navi2ch-next-property (point) 'navi2ch-link)
		  (goto-char start) nil))
	     (t (goto-char start) nil))))))

;;;
;;; $B4m81$J(B URL $B$r3+$/A0$K7Y9p$r$9$k(B.
;;;
(defvar my-navi2ch-denger-url-list nil
  "$B4m81$J(B URL $B$H$=$NFbMF$N%3%s%9%;%k$N%j%9%H(B.
$BNc(B
((\"omanchin.com\" . \"IP $BH4$-(B\"))")
(defadvice navi2ch-browse-url-internal
  (around check-url activate)
  (let ((url (ad-get-arg 0))
	(regexp (regexp-opt
		 (mapcar 'car
			 my-navi2ch-denger-url-list))))
    (if (or (not my-navi2ch-denger-url-list)
	    (not (string-match regexp url))
	    (y-or-n-p
	     (concat "$B3+$3$&$H$7$F$$$k(B URL $B$O4m81$J2DG=@-$,$"$j$^$9(B."
		     "\nURL: " url
		     "\n$BFbMF(B: "
		     (let* ((str (match-string 0 url))
			    (mes (cdr
				   (assoc str my-navi2ch-denger-url-list))))
		       (if mes mes "???"))
		     "\n$B3+$-$^$9$+!)(B ")))
	ad-do-it
      (message "$BCf;_$7$^$7$?(B."))))



(provide 'my-navi2ch)
